-- Specjalizazcja dla Bizon Z020
-- 03.10.2019
-- Rockstar94


Z020Combine = {};

function Z020Combine.prerequisitesPresent(specializations)
    return	true
end

function Z020Combine.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "playerTriggerCheck", Z020Combine.playerTriggerCheck)
	SpecializationUtil.registerFunction(vehicleType, "playerTriggerAttacherCheck", Z020Combine.playerTriggerAttacherCheck)
	SpecializationUtil.registerFunction(vehicleType, "lowerImplementDashboardState", Z020Combine.lowerImplementDashboardState)
	SpecializationUtil.registerFunction(vehicleType, "pipeUnloadDashboardState", Z020Combine.pipeUnloadDashboardState)
	SpecializationUtil.registerFunction(vehicleType, "actionEventTextUpdate", Z020Combine.actionEventTextUpdate)
end

function Z020Combine.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", Z020Combine)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", Z020Combine);
	SpecializationUtil.registerEventListener(vehicleType, "onDelete", Z020Combine);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", Z020Combine)
	SpecializationUtil.registerEventListener(vehicleType, "onDraw", Z020Combine)
	SpecializationUtil.registerEventListener(vehicleType, "onPreAttach", Z020Combine)
	SpecializationUtil.registerEventListener(vehicleType, "onPreDetach", Z020Combine)
end

function Z020Combine.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "getIsDischargeNodeActive", Z020Combine.getIsDischargeNodeActive)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "startThreshing", Z020Combine.startThreshing)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "stopThreshing", Z020Combine.stopThreshing)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "handleDischargeRaycast", Z020Combine.handleDischargeRaycast)
end

function Z020Combine:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
	if self.spec_pipe.actionEvents[InputAction.TOGGLE_PIPE] ~= nil then
		self:removeActionEvent(self.spec_pipe.actionEvents, InputAction.TOGGLE_PIPE);
	end
	if self.spec_cylindered.actionEvents[InputAction.Z020_ATTACHER_UP_DOWN] ~= nil then
		self:removeActionEvent(self.spec_cylindered.actionEvents, InputAction.Z020_ATTACHER_UP_DOWN);
	end
	if self.spec_cylindered.actionEvents[InputAction.Z020_ATTACHER_ROT] ~= nil then
		self:removeActionEvent(self.spec_cylindered.actionEvents, InputAction.Z020_ATTACHER_ROT);
	end
end;

function Z020Combine:onLoad(savegame)
	self.levers = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.levers#rootNode"));
	self.leversNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.levers#node"));
	setVisibility(self.levers, false)
	
	self.playerTrigger = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.pipe.playerTrigger#node"));
	addTrigger(self.playerTrigger, "playerTriggerCheck", self); 
	self.playerInTrigger = false;
	
	self.actionEvents = {};
    if self.loadDashboardsFromXML ~= nil then
        self:loadDashboardsFromXML(self.xmlFile, "vehicle.cylindered.dashboards", {valueTypeToLoad = "lowerImplement", valueObject = self, valueFunc = Z020Combine.lowerImplementDashboardState, minFunc = -1, maxFunc = 1, additionalAttributesFunc = Cylindered.movingToolDashboardAttributes, idleValue = 0})
		self:loadDashboardsFromXML(self.xmlFile, "vehicle.cylindered.dashboards", {valueTypeToLoad = "pipeUnload", valueObject = self, valueFunc = Z020Combine.pipeUnloadDashboardState, minFunc = 0, maxFunc = 1, additionalAttributesFunc = Cylindered.movingToolDashboardAttributes, idleValue = 0})
	end

	self.tipToObject = false
	
	self.playerTriggerAttacher = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.attacher#trigggerNode"));
	addTrigger(self.playerTriggerAttacher, "playerTriggerAttacherCheck", self); 
	
    if self.isClient then	
		self.leversSupportedMods = {};
		local i = 0;
		while true do
			local baseString = string.format("vehicle.levers.supportedMod(%d)#",i);
			local modString = getXMLString(self.xmlFile,baseString.."modString");
			local author = getXMLString(self.xmlFile,baseString.."author");
			if modString == nil then
				break;
			end;
			local supportedMod = {};
			supportedMod.modString = modString
			supportedMod.author = author;
			local linkedToCab =	getXMLString(self.xmlFile,baseString.."linkedToCabSuspension");
			local linkedToCabSuspensionIndexNumber = getXMLFloat(self.xmlFile,baseString.."linkedToCabSuspensionIndexNumber");
			if linkedToCab ~= nil then
				supportedMod.linkedToCab = linkedToCab
				supportedMod.linkedToCabSuspensionIndexNumber = linkedToCabSuspensionIndexNumber
			end
			local translation = getXMLString(self.xmlFile,baseString.."translation");
			if translation ~= nil then
				local x,y,z = StringUtil.getVectorFromString(translation);
				supportedMod.translation = {x,y,z}
			end

			table.insert(self.leversSupportedMods, supportedMod);
			i=i+1;
		end
	end
	
	
	self.handNode1 = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.levers#handNode1"));
	self.handNode2 = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.levers#handNode2"));
end

function Z020Combine:onDelete()
	removeTrigger(self.playerTrigger)
	removeTrigger(self.playerTriggerAttacher)
	
	if self.supportedModAttached then
		unlink(self.levers);
	end
end

function Z020Combine:playerTriggerCheck(triggerId, otherId, onEnter, onLeave, onStay)
    if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
        self.playerInTrigger = true;
		_, self.actionEventId = self:addActionEvent(self.actionEvents, InputAction.TOGGLE_PIPE, self, Pipe.actionEventTogglePipe, false, true, false, true, nil);
		self:actionEventTextUpdate()
		if self.configurations["vehicleType"] == 1 then
		_, self.actionEventUnloadId  = self:addActionEvent(self.actionEvents, InputAction.IMPLEMENT_EXTRA2, self, Dischargeable.actionEventToggleDischargeToGround, false, true, false, true, nil);
		_, self.actionEventUnloadToObjectId = self:addActionEvent(self.actionEvents, InputAction.IMPLEMENT_EXTRA4, self, Z020Combine.tipToObjectCallback, false, true, false, true, nil);
		end
    elseif onLeave and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
        self.playerInTrigger = false;
		self:removeActionEvent(self.actionEvents, InputAction.TOGGLE_PIPE);
		if self.configurations["vehicleType"] == 1 then
			self:removeActionEvent(self.actionEvents, InputAction.IMPLEMENT_EXTRA2);
			self:removeActionEvent(self.actionEvents, InputAction.IMPLEMENT_EXTRA4);
		end
    end;
end;

function Z020Combine:playerTriggerAttacherCheck(triggerId, otherId, onEnter, onLeave, onStay)
    if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		_, self.actionEventAttacherId = self:addActionEvent(self.actionEvents, InputAction.Z020_ATTACHER_UP_DOWN, self, Z020Combine.attacherUpDown, false, false, true, true, nil);
		_, self.actionEventAttacherRotId = self:addActionEvent(self.actionEvents, InputAction.Z020_ATTACHER_ROT, self, Z020Combine.attacherRot, false, false, true, true, nil);
    elseif onLeave and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		self:removeActionEvent(self.actionEvents, InputAction.Z020_ATTACHER_UP_DOWN);
		self:removeActionEvent(self.actionEvents, InputAction.Z020_ATTACHER_ROT);
    end;
end;

function Z020Combine.attacherUpDown(self, actionName, inputValue, callbackState, isAnalog, isMouse)
	self.spec_cylindered.movingTools[1].move = inputValue
	local _,y,_ = getTranslation(self.spec_cylindered.movingTools[1].node)
	local zzz = y * 100 / self.spec_cylindered.movingTools[1].transMax
	g_inputBinding:setActionEventText(self.actionEventAttacherId, g_i18n:getText("action_z020_attacherPosition") .. ": " .. string.format("%d", zzz) .. "%")
end

function Z020Combine.attacherRot(self, actionName, inputValue, callbackState, isAnalog, isMouse)
	self.spec_cylindered.movingTools[2].move = inputValue
	local _,y,_ = getRotation(self.spec_cylindered.movingTools[2].node)
	local zzz = self.spec_cylindered.movingTools[2].rotMax-self.spec_cylindered.movingTools[2].rotMin
	local yyy = y * 10 /zzz
	local xxx = yyy - 80
	local www = xxx * 10
	g_inputBinding:setActionEventText(self.actionEventAttacherRotId, g_i18n:getText("action_z020_attacherRotation") .. ": " .. string.format("%d", www) .. "%")
end

function Z020Combine:onUpdate(dt)
	if g_gui.currentGuiName == "ShopConfigScreen" then
		if g_currentMission.shopController.buyItemFilename ~= nil then
			if g_currentMission.shopController.buyItemFilename == self.configFileName then
				if g_currentMission.shopController.isBuying then
					if g_currentMission.shopController.buyVehicleNow > 0 then
						self.loadInfoDialog = true
					end
					if self.loadInfoDialog then
						if self.lockExistingDialog == nil then
							g_gui:showInfoDialog({text=g_i18n:getText("shop_z020_additionalDialog"), dialogType=DialogElement.TYPE_BENCHMARK})
							if g_currentMission.shopController.buyVehicleNow == 0 then
								self.lockExistingDialog = true
							end
						end
					end
				end
			end
		end
	end

	self.actionStartTip = false
	self.actionStopTip = false
	if self.playerInTrigger then
		self:actionEventTextUpdate()
		if self.configurations["vehicleType"] == 1 then
			if self.spec_pipe.targetState == self.spec_pipe.numStates then
				if self:getFillUnitFillLevel(1) > 0 then
					if self:getIsTurnedOn() == false then
						g_currentMission:addExtraPrintText(g_i18n:getText("info_z020_turnOnVehicleToUnlod"));
					end;
					if self.spec_dischargeable.currentDischargeState == Dischargeable.DISCHARGE_STATE_OFF then
						local currentDischargeNode = self.spec_dischargeable.currentDischargeNode
						if self:getIsDischargeNodeActive(currentDischargeNode) then
							if self:getCanDischargeToGround(currentDischargeNode) and self:getCanToggleDischargeToGround() then
								self.actionStartTip = true
								g_inputBinding:setActionEventText(self.actionEventUnloadId, g_i18n:getText("action_startTipToGround"))
							end
						end
					elseif self.spec_dischargeable.currentDischargeState == Dischargeable.DISCHARGE_STATE_GROUND then
						self.actionStopTip = true
						g_inputBinding:setActionEventText(self.actionEventUnloadId, g_i18n:getText("action_stopTipToGround"))
					end;
				end
			end
		end
	end
	
	if self.configurations["vehicleType"] == 1 then
		if self.spec_dischargeable.actionEvents[InputAction.TOGGLE_TIPSTATE_GROUND] ~= nil then
			self:removeActionEvent(self.spec_dischargeable.actionEvents, InputAction.TOGGLE_TIPSTATE_GROUND);
		end
		g_inputBinding:setActionEventActive(self.actionEventUnloadId, self.actionStartTip or self.actionStopTip)
		g_inputBinding:setActionEventActive(self.actionEventUnloadToObjectId, self.dischargeToObjectAction and self.spec_pipe.targetState == self.spec_pipe.numStates and self:getIsTurnedOn() == true)
	end
	
    if self.isClient then
		if not self:getRootVehicle():getIsActive() then
			self:raiseActive()
		end
	end
end

function Z020Combine:handleDischargeRaycast(superFunc, dischargeNode, hitObject, hitShape, hitDistance, hitFillUnitIndex, hitTerrain)
	self.dischargeToObjectAction = false
    local stopDischarge = false
    if hitObject ~= nil then
		local fillType = self:getDischargeFillType(dischargeNode)
		local allowFillType = hitObject:getFillUnitAllowsFillType(hitFillUnitIndex, fillType)
		if allowFillType and hitObject:getFillUnitFreeCapacity(hitFillUnitIndex, fillType, self:getOwnerFarmId()) > 0 then
			self.dischargeToObjectAction = true
			if self.configurations["vehicleType"] == 1 then
				if self.tipToObject then
					self:setDischargeState(Dischargeable.DISCHARGE_STATE_OBJECT, true)
					g_inputBinding:setActionEventText(self.actionEventUnloadToObjectId, g_i18n:getText("action_stopOverloading"))
				else
					stopDischarge = true
					self.tipToObject = false
					local objectName = XMLUtil.getXMLI18NValue(hitObject.xmlFile, "vehicle.storeData", getXMLString, "brand", nil, hitObject.customEnvironment, true)
					local objectBrand = XMLUtil.getXMLI18NValue(hitObject.xmlFile, "vehicle.storeData", getXMLString, "name", nil, hitObject.customEnvironment, true)
					local fullName = (objectName .. " " .. objectBrand)
					g_inputBinding:setActionEventText(self.actionEventUnloadToObjectId, string.format(g_i18n:getText("action_z020_startOverloadingToObject"), fullName))
				end
			else
				self:setDischargeState(Dischargeable.DISCHARGE_STATE_OBJECT, true)
			end;
		else
			stopDischarge = true
			self.tipToObject = false
		end
    else
        stopDischarge = true
		self.tipToObject = false
    end
    if stopDischarge and self:getDischargeState() == Dischargeable.DISCHARGE_STATE_OBJECT then
        self:setDischargeState(Dischargeable.DISCHARGE_STATE_OFF, true)
    end
end

function Z020Combine:onDraw()
	if self.configurations["vehicleType"] == 2 then
		if self.spec_pipe.targetState == self.spec_pipe.numStates then
			if self:getFillUnitFillLevel(1) > 0 then
				if self:getIsTurnedOn() == false then
					g_currentMission:addExtraPrintText(g_i18n:getText("info_z020_turnOnVehicleToUnlod"));
				end;
			end
		end
	end
end

function Z020Combine:getIsDischargeNodeActive(superFunc, dischargeNode)
    if dischargeNode.index == self:getPipeDischargeNodeIndex() then
        return self.spec_pipe.unloadingStates[self.spec_pipe.currentState] == true and self:getIsTurnedOn() == true
    end
    return superFunc(self, dischargeNode)
end

function Z020Combine.tipToObjectCallback(self, actionName, inputValue, callbackState, isAnalog, isMouse)
	self.tipToObject = not self.tipToObject
end

function Z020Combine:lowerImplementDashboardState()	
	if self.spec_attacherJoints.attachedImplements[1] ~= nil then
		self.implementMove = 1 - self.spec_attacherJoints.attacherJoints[1].moveAlpha 
		if self.spec_attacherJoints.attacherJoints[1].moveAlpha < 0.8 then
			self.implementMove = self.spec_attacherJoints.attacherJoints[1].moveAlpha 
		end
		if self.spec_attacherJoints.attachedImplements[1].object:getIsLowered() == true then
			return self.implementMove * 7
		elseif self.spec_attacherJoints.attachedImplements[1].object:getIsLowered() == false then
			return -self.implementMove * 7
		end
	end
	return 0
end

function Z020Combine:pipeUnloadDashboardState()
	return self.spec_pipe.animationNodes[1].currentAlpha
end

function Z020Combine:actionEventTextUpdate()
	if self.spec_pipe.targetState == self.spec_pipe.numStates then
		g_inputBinding:setActionEventText(self.actionEventId, g_i18n:getText("action_pipeIn"))
	else
		g_inputBinding:setActionEventText(self.actionEventId, string.format(g_i18n:getText("action_pipeOut"), ""))
	end;
end

function Z020Combine:startThreshing()
    if self.spec_combine.numAttachedCutters > 0 then
        for _,cutter in pairs(self.spec_combine.attachedCutters) do
            cutter:setIsTurnedOn(true, true)
        end
        if self.spec_combine.threshingStartAnimation ~= nil and self.playAnimation ~= nil then
            self:playAnimation(self.spec_combine.threshingStartAnimation, self.spec_combine.threshingStartAnimationSpeedScale, self:getAnimationTime(self.spec_combine.threshingStartAnimation), true)
        end
        if self.isClient then
            g_soundManager:stopSamples(self.spec_combine.samples)
            g_soundManager:playSample(self.spec_combine.samples.start)
            g_soundManager:playSample(self.spec_combine.samples.work, 0, self.spec_combine.samples.start)
        end
        SpecializationUtil.raiseEvent(self, "onStartThreshing")
    end
end

function Z020Combine:stopThreshing()
    if self.isClient then
        g_soundManager:stopSamples(self.spec_combine.samples)
        g_soundManager:playSample(self.spec_combine.samples.stop)
    end
    self:setCombineIsFilling(false, false, true)
    for cutter,_ in pairs(self.spec_combine.attachedCutters) do
        cutter:setIsTurnedOn(false, true)
    end
    if self.spec_combine.threshingStartAnimation ~= nil and self.spec_combine.playAnimation ~= nil then
        self:playAnimation(self.spec_combine.threshingStartAnimation, -self.spec_combine.threshingStartAnimationSpeedScale, self:getAnimationTime(self.spec_combine.threshingStartAnimation), true)
    end
    SpecializationUtil.raiseEvent(self, "onStopThreshing")
end

function Z020Combine:onPreAttach()
	self.supportedModAttached = nil
	self.supportedModAttachedTranslation = nil
	
	if self.spec_attachable.attacherVehicle.customEnvironment ~= nil then
		local attacherModName = XMLUtil.getXMLI18NValue(self.spec_attachable.attacherVehicle.xmlFile, "vehicle.storeData", getXMLString, "name", nil, self.spec_attachable.attacherVehicle.customEnvironment, true)
		local attachedModAuthor = getXMLString(loadXMLFile("modDesc", g_modsDirectory .. "/" .. self.spec_attachable.attacherVehicle.customEnvironment .. "/" .. "modDesc.xml"), "modDesc.author");
		
		for _,supportedMod in ipairs(self.leversSupportedMods) do
			if supportedMod.modString == attacherModName and supportedMod.author == attachedModAuthor then
				if supportedMod.linkedToCab then
					link(self.spec_attachable.attacherVehicle.spec_suspensions.suspensionNodes[supportedMod.linkedToCabSuspensionIndexNumber].node, self.levers);
				else
					link(self.spec_attachable.attacherVehicle.rootNode, self.levers);
				end
				setVisibility(self.levers, true)
				self.supportedModAttached = true
				if supportedMod.translation ~= nil then
					setTranslation(self.leversNode , unpack(supportedMod.translation))
					self.supportedModAttachedTranslation = true
				end;
			end
		end;
	end
end

function Z020Combine:onPreDetach()
	if self.supportedModAttached then
		if self.supportedModAttachedTranslation then
			setTranslation(self.leversNode ,-0.395,0.423,0)
		end;
		unlink(self.levers);
		setVisibility(self.levers, false)
	end
end